;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 43A4			;17316

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Laddr=09D8			;Line address subroutine
@Vars=4010
@DFile=400C			;Address of display file
@LastK=4025			;Keyboard debounce value
@NxtLine=4029			;Next program line to be executed
@PRBuff=403C			;16444
@MainForm=4083			;Stored Mainform address

;================================
; Jump Table
;================================
	JP KeyLoop		;17316, Key loop and sorter
	JR LoadMain		;17319, Load Mainform
	JR SaveMain		;17321, Save Mainform
	JP FileDrop		;17323, File dropdown menu
	JP HelpDrop		;17326, Help dropdown menu
	JP LexiA		;17329, Lexicon A dropdown menu
	JP LexiG		;17332, Lexicon G dropdown menu
	JP LexiM		;17335, Lexicon M dropdown menu
	JP LexiS		;17338, Lexicon S dropdown menu
	JP LexiT		;17341, Lexicon T dropdown menu
	JP User			;17344, User dropdown menu
        JP BMP			;17347, BMP dropdown menu
;================================
@PageA
	9D			;LexiA page number
@PageG
	9D			;LexiG page number
@PageM
	9D			;LexiM page number
@PageS
	9D			;LexiS page number
@PageT
	9D			;LexiT page number
@PageU
	9D			;User page number
@PageB
	9D			;BMP page number
@Selected
	00 00			;DFile address of selected pattern
;============================
; Load Mainform
;============================
@LoadMain
	PUSH HL
	LD HL,MainForm		;Set source
	LD DE,(DFile)		;Set destination
	JR +7
;============================
; Save Mainform
;============================
@SaveMain
	PUSH HL
	LD HL,(DFile)		;Set source
	LD DE,MainForm		;Set destination
	LD BC,0319		;BC=byte count
	LDIR			;Copy source to desination
	POP HL
	RET			;Return 0
;============================
; Wait for key release
;============================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	LD BC,0000
	RET
;================================
; Print dropdown menu
;================================
@DropDown
	CALL SaveMain		;Save Mainform
	LD HL,(NxtLine)
@DropMenu
	LD BC,0005		;Skip REM header
	ADD HL,BC		;Start of data
	CALL Release		;Wait for key release
	LD B,(HL)		;Get Ycoord
	INC HL
	LD C,(HL)		;Get Xcoord
	INC HL			;Start of print data
	PUSH HL			;PUSH datapointer
	CALL PrntPos		;Get print position of the NW corner of dropdown
	POP BC			;POP Datapointer into BC
@DropY
	LD (Drop+1),HL		;Save start of row addr
@DropLoop
	LD A,(BC)		;Get chr to print
	CP 76			;EO REM?
	RET Z			;Return printing done
	CP 0C			;EOL marker ()?
	JR Z Drop		;Jump if EOL
	LD (HL),A		;Print A
	JR +7
@Drop
	LD HL,0000		;Get start of row addr
	ADD HL,DE		;Next row addr
	INC BC			;Increase data pointer
	JR DropY		;Start of row addr
	INC HL			;Increase print position
	INC BC			;Increase data pointer
	JR DropLoop
;================================
; File menu dropdown
;================================
@FileDrop
	CALL DropDown		;Print file dropdown menu
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD HL,FileKeys		;Valid keys table
	LD BC,0014 		;Byte count
	CPIR 			;Locate key
	JR NZ +1		;Jump if no match
	INC B			;B=Hi byte of line number
@SetLine
	LD C,(HL)
	LD H,B
	LD L,C
	CALL Release
	CALL Laddr
	LD (NxtLine),HL
	JP LoadMain
;================================
@FileKeys			;Key code/lo byte of line number
	"G" EA			;490 Gen Count
	"B" 2C			;300 B<->W Swap
	"S" 36			;310 SLOW/FAST
	"P" 40			;320 Petridish
	"I" 4A			;330 Invert BMP
	"V" 54			;340 VFlip BMP
	"U" 5E			;350 Upsize
	"F" 68			;360 Frames set-up
	"D" D6			;470 Slide show
	"L" F4			;500 LOAD BMP
	5A			;EOTable marker=GOTO 90
;================================
; Lexicon dropdowns, Part 1
; 
; Set dropdown parameters
;================================
@LexiA				;LexiA parameters
	LD B,0C			;PageA-0C=LoByte of REM Line
	LD C,+20		;Offset from (VARS) to page number
	LD A,+41		;Offset from (DFILE) to start of menu-33
	LD HL,PageA		;LexiA page number address
	JR SetPars
@LexiG				;LexiG parameters
	LD B,02			;PageG-02=LoByte of REM Line
	LD C,+18		;Offset from (VARS) to page number
	LD A,+43		;Offset from (DFILE) to start of menu-33
	LD HL,PageG		;LexiG page number address
	JR SetPars
@LexiM				;LexiM parameters
	LD B,F8			;PageM-F8=LoByte of REM Line
	LD C,+16		;Offset from (VARS) to page number
	LD A,+45		;Offset from (DFILE) to start of menu-33
	LD HL,PageM		;LexiM page number address
	JR SetPars
@LexiS				;LexiS parameters
	LD B,EE			;PageS-EE=LoByte of REM Line
	LD C,+14		;Offset from (VARS) to page number
	LD A,+47		;Offset from (DFILE) to start of menu-33
	LD HL,PageS		;LexiS page number address
	JR SetPars
@LexiT				;LexiT parameters
	LD B,E4			;PageT-E4=LoByte of REM Line
	LD C,+12		;Offset from (VARS) to page number
	LD A,+49		;Offset from (DFILE) to start of menu-33
	LD HL,PageT		;LexiT page number address
	JR SetPars
@User				;User parameters
	LD B,DA			;PageU-DA=LoByte of REM Line
	LD C,+10		;Offset from (VARS) to page number
	LD A,+51		;Offset from (DFILE) to start of menu-33
	LD HL,PageU		;LexiU page number address
	JR SetPars
@BMP				;BMP parameters
	LD B,D0			;PageB-D0=LoByte of REM Line
	LD C,+5			;Offset from (VARS) to page number
	LD A,+56		;Offset from (DFILE) to start of menu-33
	LD HL,PageB		;BMP page number address
@SetPars
	LD (MenuStart+1),A
	LD A,B
	LD (LoByte+1),A
	LD A,C
	LD (PageNum+1),A
	LD (GetPage+1),HL
	LD (SetPage+1),HL
;================================
; Lexicon dropdowns, Part 2
; 
; Print dropdown
;================================
@LexDrop
	CALL DropDown		;Print Screens dropdown
	LD H,01 		;Hi byte of REM line
@GetPage
	LD A,(0000)		;Get current page number
@LoByte
	SUB 0C			;Adjust page number 
	LD L,A			;Lo byte of REM line
	CALL LAddr		;Get line address
	CALL DropMenu		;Print dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "A"			;Key<"A"?
	JR NC TestW		;Jump if key<"A"
@WrongKey
	JP LoadMain		;Return via LoadMain BC=0000
@TestW
	CP "W"			;Key>="W"?
	JR NC WrongKey		;Jump if key>"V"
	SUB 25 			;A=0 to 22
	LD D,A 			;Save A			
	CALL Release		;Wait for key release
	LD A,D 			;Restore A
	CP 16			;A="V"
	JR NZ +14		;Jump if not "V"
	LD HL,(Vars)		;Locate page number
@PageNum
	LD BC,0013
	SBC HL,BC 
	LD A,(HL) 		;Get page number
@SetPage
	LD (0000),A 		;Set page number
	JR LexDrop+3		;Print next page
	LD HL,(DFile)
@MenuStart
	LD BC,0000		;Menu start offset
	ADD HL,BC		;Menu start address
	LD B,D			;B=Item number
@AddRow
	LD DE,0021		;Bytes/row
	ADD HL,DE		;Add row count to start address
	DJNZ AddRow
	LD DE,PRBuff		;Set destination
	LD C,09			;Counter
	LDIR			;Copy file name to PRBuff
	CALL LoadMain		;Restore Mainform
	INC C
	RET			;Return BC=0001
;================================
; Help menu dropdown
;================================
@HelpDrop
	CALL DropDown		;Print help dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD H,02			;Hi byte of GOTO line number
	CP "R"			;"R"eadme
	JR NZ +5
	LD L,94			;GOTO 660
@HelpLine
	JP SetLine+3
	CP "L"			;"L"ife rules
	JR NZ +4
	LD L,80			;GOTO 640
	JR HelpLine
	CP "K"			;"K"ey Guide
	JR NZ +4
	LD L,6C			;GOTO 620
	JR HelpLine
	CP "A"			;"A"bout
	JR Z +6
@GOTO90
	LD HL,+90		;GOTO 90
	JP SetLine+6
;================================
; About
;================================
	LD HL,+610		;REM 610
	CALL LAddr
	LD (NxtLine),HL
	CALL LoadMain		;Reload Mainform
	CALL DropDown+3		;Print REM 610 or 620
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CALL Release		;Wait for key release
	CALL LoadMain		;Reload Mainform
	JR GOTO90
;================================
; Print Position
;  B=Ycoord
;  C=Xcoord
;  BC unchange
;  On exit HL=Print Position address
;================================
@PrntPos
	PUSH BC			;PUSH YXcoords
	LD HL,0000		;Set accumulator to 0
	LD DE,0021		;Bytes/row
	LD A,B
	AND A			;Row=0?
	JR Z +3			;Yes, move on
	ADD HL,DE		;HL=33*row value
	DJNZ -3
	ADD HL,BC		;HL=33*row value+column offset
	LD BC,(DFile)		;Get DFile
	INC BC			;Skip 1st N/L chr
	ADD HL,BC		;HL=print position
	POP BC			;POP YXcoords
	RET	
;================================
; Test for keypress.
; On exit...
;   A=chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress, D=255 if no key press
	INC D 			;0?
	RET Z
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	INC A			;Set Z flag
	LD A,(HL)
	RET			;Return chr code to BASIC
;================================
; Key loop
;================================
@KeyLoop
	CALL KeyScan		;Get key press
	JR Z KeyLoop		;No, loop until key pressed
	LD HL,TopMenu		;Valid keys table
	LD BC,001E 		;Byte count
	CPIR 			;Locate key
	JR NZ KeyLoop		;Jump if no match
	CALL Release
	LD C,(HL)		;Line number Hi byte
	INC HL
	LD B,(HL)		;Line number Lo byte
	RET			;Return GOTO line number
;================================
; Key loop
;================================
@TopMenu
	"F" 00FA		;250
	"A" 0190		;400
	"G" 019A		;410
	"M" 01A4		;420
	"S" 01AE		;430
	"T" 01B8		;440
	"U" 01C2		;450
	"B" 01CC		;460
	"H" 0258		;600
	"X" 02BC		;700
