;===========================================
;       ZX-Assembler-2 
;   by Artic Computing LTD
;===========================================
; 16x16 LIFE
; Based on LIFE by Toni Baker
; from "Mastering Machine Code on Your ZX81"
;===========================================

ORG 4CD6

@DFile=400C
@LastK=4025			;Keyboard debounce value
@Seed=4032
@Frames=4034			;Frame counter
@Gen0=8000
@Dump=8200

	JR GenZero
	JR Life
;================================
@Table
	EF 01 01 10 10 FF FF F0	;Data representing the displacements
;	??  N NE  E SE  S  W NW	;of the neighbouring squares.
				;NW-N-NE-E-SE-S-SW-W
;================================
@GenZero			;Print random GEN0
	LD C,10			;C counts the number of rows printed.
	LD HL,Gen0-1
@NextY
	LD B,10			;B counts the number of columns.
@NextX
	INC HL
	CALL Rnd8Bit
	CP C4			;196, 25% chance of printing "gO"
	JR C Black		;If CY then print black space
	LD A,B4			;"gO"
	JR Char
@Black
	LD A,80			;"g "
@Char
	LD (HL),A
	DJNZ NextX		;Loop until B=0
	INC HL
	LD (HL),76
	DEC C		
	JR NZ NextY		;Same for next row.
	JR Display
;================================
; 8-Bit Random Number Generator
; Reg BC not used
; On exit...
;  Reg A=random number in random
;        0 to 255
;================================
@Rnd8Bit
	PUSH HL	
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+2),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	POP HL
	RET
;================================
@Display
	LD HL,(DFile)
	LD BC,008D
	ADD HL,BC
	EX DE,HL		;Set destination
	LD HL,Gen0		;Set Source
	LD A,10			;Row count
@Loop
	LD C,10			;Column count
	LDIR
	INC HL			;Skip N/L
	EX DE,HL		;Swap for ADD
	LD C,11			;Offset to next row
	ADD HL,BC		;Start of next row
	EX DE,HL		;Swap pointers
	DEC A			;Decrease row counter
	JR NZ Loop		;Loop until A=0
	RET
;================================
@Life
	LD B,00			;B counts the no. of cell positions.
	LD DE,Dump		;Start of the working area used to compute the next gen.
	LD HL,Gen0		;Start of GenX-1
	PUSH HL			;PUSH start GenX
	DEC HL
@Copy
	INC HL
	LD A,(HL)		;Copy the current generation to the working space.
	CP 76
	JR Z Copy
	LD (DE),A
	INC DE
	DJNZ Copy
	LD DE,Dump		;PUSH start of dump.
	PUSH DE
@NextCell
	LD C,00			;C=Cell counter
	POP DE
	POP HL
	LD A,(HL)		;Skip N/L
	CP 76
	JR NZ Valid
	INC HL
@Valid
	PUSH HL
	EX DE,HL		;Store the position within the dump
	PUSH HL			;of the cell being examined in HL,
				;and also stack it.
	LD DE,Table		;Point DE to table of displacements.
@NexDis
	LD A,(DE)		;Find displacement.
	CP 0E			;If this "displacement" is 0E we
	JR Z Counted		;have reached the end of the table.
	INC DE			;Point DE to next item in table.
	ADD A,L			;Find neighbouring cell-position.
	LD L,A
	LD A,(HL)		;Is there a cell there?
	CP B4
	JR NZ NexDis
	INC C			;Increase cell count
	JR NexDis
@Counted
	POP HL			;Retrieve cell position.
	LD A,C
	CP 02			;Are there less than two neighbours?
	JR C Death		;If so no cell appears.
	CP 04			;Are there four or more?
	JR NC Death		;If so no cell appears.
	CP 03			;Are there precisely three?
	JR Z Birth		;If so, a cell does appear.
	LD A,(HL)
	JR Put
@Birth
	LD A,B4
	JR Put
@Death
	LD A,80			;A now contains the right character.
@Put
	EX (SP),HL		;Retrieve print position.
	LD (HL),A		;Print character.
	INC HL			;Move print position along line.
	EX (SP),HL		;Retrieve cell-position.
	INC HL			;Look at next cell-position.
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Abort		;Return if keypressed
	PUSH HL			;Stack the position.
	LD A,L			;Check the value of L to find out
	AND A			;whether or not we have printed the
	JR NZ NextCell		;last cell-position.
	POP HL			;Restore the stack
	POP HL
	CALL Display
	JR Life
@Abort
	POP HL			;Restore the stack
	RET			;Return to BASIC