;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;           32x24 LIFE
;      Gregory C. Harder, 2019
;
;Based on the LIFE by J.R. Ball from
;"Michael Owen's Cassette 4" 1983
;================================

ORG 467B			;18043

@KeyScan=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Workarea=8000			;Workarea for Generation analysis
@DFCopy=8021			;Copy of DFile contained in workarea
@RiteEdge=8041			;Workarea right edge address
@DFile=400C			;Start of display file
;==============================
; Jump Table
;==============================
	JR Petridish		;18043 Petridish life single step
        JR PetriLoop		;18045 Petridish life continuous loop
	JP Global		;18047 Global life single step
	JP GLoop		;18050 Global life continuous loop
	JP Clear		;18053 Clear the workarea
	JP WonB			;18056 White O on black
	JP BonW			;18059 Black O on white
;==============================
; Analyze the neighborhood and return cell count
;==============================
@Analyze
	PUSH BC 		;PUSH row counter
	PUSH HL 		;PUSH cell addr.
	LD DE,0021 		;Row length + N/L
;	AND A 			;Clear CY, not needed
	SBC HL,DE 		;N cell
	LD A,76 		;N/L char.
	LD BC,0000 		;Row offset 0000
	INC (HL) 		;N cell value
	DEC HL 			;NW cell
	CP (HL) 		;N/L?
	JR NZ +3		;Jump if not N/L
	LD C,20 		;Yes
	ADD HL,BC 		;Wrap right
	INC (HL) 		;NW cell value
	ADD HL,DE 		;W cell
	INC (HL) 		;W cell value
	ADD HL,DE 		;SW cell
	INC (HL) 		;SW cell value
	SBC HL,BC 		;BC=00 or 20 if wrap right 
	LD C,B	 		;Row offset 0000
	INC HL 			;S cell
	INC (HL) 		;S cell value
	INC HL 			;SE cell
	CP (HL) 		;N/L?
	JR NZ +4		;Jump if not N/L
	LD C,20 		;Yes
	SBC HL,BC 		;Wrap left
	INC (HL) 		;SE cell value
	AND A			;Clear CY
	SBC HL,DE 		;E cell
	INC (HL) 		;E cell value
	SBC HL,DE 		;NE cell
	INC (HL) 		;NE cell value
	POP HL 			;POP cell addr.
	POP BC 			;POP row counter
	RET 
;==============================
; Petridish LIFE single step
;==============================
@Petridish			;Petridish life
	LD HL,Update-1		;Set this instruction's value to C9=RET
	LD (HL),C9		;Set to RET instruction
	PUSH HL			;PUSH address
	CALL Global		;Run the LIFE algorithm
	POP HL			;POP address
	LD (HL),B		;Set to NOP
	LD A,18			;Clear the left & right edges.
	LD HL,DFCopy+1		;Start of left edge
	LD DE,RiteEdge		;Start of right edge
	LD BC,0021		;Bytes/row
@EdgeClear
	LD (HL),00		;Set left edge to 00/80
	ADD HL,BC		;Next left edge row
	EX DE,HL		;Swap HL,DE
	LD (HL),00		;Set right edge to 00/80
	ADD HL,BC		;Next right edge row
	DEC A			;Decrease counter
	JR NZ EdgeClear		;Loop until A=0
	JR PlotCells		;Plot results
;==============================
; Petridish loop until keypress
;==============================
@PetriLoop
	CALL Petridish		;Run one PetriDish gen
	CALL GetKey		;Scan for key press
	CP 01			;No keys pressed?
	LD BC,0000		;Set return value
	RET NZ			;RET to BASIC if key pressed
	JR PetriLoop		;Next Gen
;==============================
; Setup workarea and DFCopy
;==============================
@SetUp
	LD A,76			;A=EOF markers
	LD HL,Workarea		;HL=Workarea	
	LD D,H
	LD E,L			;DE=Workarea	
	LD (HL),A 		;Set BOF marker
	LD BC,035A		;Offset to last byte of workarea
	ADD HL,BC		;HL=Last byte of workarea
	LD (HL),A		;Set EOF marker
	LD DE,DFCopy
	LD HL,(DFile)		;HL=DFile
	LD BC,0319 		;Byte count
	LDIR 			;Copy DFile to DFCopy
	RET
;==============================
; Global LIFE single step
;==============================
@Global
	CALL SetUp
	LD HL,DFCopy
	LD B,18 		;Row counter
@GlobeLoop
	INC HL 			;Next byte in row
	BIT 6,(HL) 		;N/L?
	JR NZ +7		;Jump if N/L
	BIT 5,(HL) 		;"O" char?
	CALL NZ Analyze		;Yes, analyze cell
	JR GlobeLoop
	DJNZ GlobeLoop		;Loop until B=0
	NOP			;Self modifying code, value=C9 or 00
@Update				;Update workarea rows 24 then 0
	LD DE,DFCopy		
	LD B,20 		;Bytes in row counter            
	LD DE,Workarea		;DE=Workarea
	LD HL,0318 		;DFSize
	ADD HL,DE 		;
@Row24Loop
	INC HL 			;HL=row 24
	INC DE 			;DE=row 0
	LD A,(DE) 		;A=(row 0)
	ADD A,(HL) 		;A=(row 0)+(row 24) analysis
	LD (HL),A 		;Set row 24 analysis
	XOR A			;A=0
	LD (DE),A 		;Clear Row 0
	DJNZ Row24Loop		;Loop until B=0
	LD DE,DFCopy
	LD HL,0318 		;DFSize
	ADD HL,DE 		;
	EX DE,HL 		;
	LD B,20 
@Row1Loop
	INC HL 			;HL=row 1
	INC DE 			;DE=row 25
	LD A,(DE) 		;A=(row 25)
	ADD A,(HL) 		;A=(row 25)+(row 1)
	LD (HL),A 		;Set row 1 analysis
	XOR A			;A=0
	LD (DE),A 		;Clear Row 25
	DJNZ Row1Loop
@PlotCells			;Plot cells based on values stored in DFCopy
	LD HL,DFCopy
	INC HL 			;HL=row 1
	LD B,18			;Row count 
@CellLoop
	LD A,(HL)		;Analysed cell value
@CellPars
	CP 03			;3 neighbors, if off turn on
	JR Z Birth
	CP 36			;"Q"=2 neightbors, if on stay on
	JR Z Birth
	CP 37			;"R"=3 neightbors, if on stay on
	JR Z Birth
@Death
	LD A,00			;00/80
	JR +2
@Birth				;Birth
	LD A,"O"		;"O" or "gO"
	LD (HL),A		;Print cell
	INC HL 			;Next DFCopy
	BIT 6,(HL) 		;EOL?
	JR Z CellLoop		;No, loop until EOL
	INC HL 			;Skip EOL marker
	DJNZ CellLoop		;Loop until B=0
	LD HL,DFCopy		;Copy DFCopy to DFile
	LD DE,(DFile)
	LD BC,0319
	LDIR 			
	RET			;Return (BC=0000)
;==============================
; Global loop
;==============================
@GLoop
	CALL Global		;Run one Global gen
	CALL GetKey		;Scan for key press
	CP 01			;No keys pressed?
	LD BC,000A		;Set return value, GOTO 10
	RET NZ			;RET to BASIC if key pressed
	JR GLoop		;Next Gen
;==============================
; Keyboard decode
;==============================
@GetKey				;Keyboard SCAN
	CALL Keyscan		;Scan the Keyboard
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL Decode		;Decode BC into ZX81 character codes
	LD A,(HL) 		;A=CHR CODE
	RET
;==============================
; Clear the workarea
;==============================
@Clear				;Clear the workarea
	LD HL,Workarea		;Set source
	LD (HL),00		;Fill with 00/80
	LD DE,Workarea+1	;Set destination
	LD BC,0359		;Byte count
	LDIR			;Fill workarea with 0s
	RET
;==============================
; Convert Gen0 to White Ink on Black Paper
;==============================
@WonB
	LD BC,0318		;Invert DFile
	LD HL,(DFILE)
	INC HL
	LD A,(HL)
	CP 76
	JR Z +2
	SET 7,(HL)
	DEC BC 
	LD A,B 
	OR C 
	JR NZ -13
;==============================
; White on Black parameters
;==============================
	LD A,80			;"g "
	LD (Clear+4),A
	LD (EdgeClear+1),A
	LD (EdgeClear+5),A
	LD (Death+1),A
	LD A,B4			;"gO"
	LD (Birth+1),A
	LD A,83
	LD (CellPars+1),A
	LD A,B6			;"gQ"
	LD (CellPars+5),A
	INC A			;"gR"
	LD (CellPars+9),A
	RET
;==============================
; Black on White parameters
;==============================
@BonW
	XOR A			;" "
	LD (Clear+4),A
	LD (EdgeClear+1),A
	LD (EdgeClear+5),A
	LD (Death+1),A
	LD A,34			;"O"
	LD (Birth+1),A
	LD A,03
	LD (CellPars+1),A
	LD A,36			;"Q"
	LD (CellPars+5),A
	INC A			;"R"
	LD (CellPars+9),A
	RET
