;========================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;========================================
; 64x48 LIFE
; Based on "Macro-LIFE" by Toni Baker
; from ZX Computing, 10/1986 p.54
;========================================

ORG 47FF			;18431

@Versn=4009			;16393 Border/No border flag
@DFile=400C
@LastK=4025			;Keyboard debounce value
@GenCopy=8000			;Start of GenCopy
@TopRow=8000			;VFlip-top row address
@BotRow=8002			;VFlip-bottom row address
@RowSave=8004			;VFlip-Line storage
@GenX=9000			;Start of GenX
@HRDFILE=A000			;Start HRDFile

	JR Gen0			;18431
	JR LIFE			;18433
	JP BMPHdr		;18435
	JP InvBMP		;18438
	JP FlipBMP		;18441
	JP Invert64		;18444
	JP Invert128		;18447
	00 00 00		;18450
	JP VFlip64		;18453
	JP VFlip128		;18456
	00 00 00		;18459
	JP Upsize		;18462
;========================================
; Prep Gen0 for use
;========================================
@Gen0
	CALL Invert64		;Invert BMP
	LD A,(Versn)		;Get Petridish flag
	LD (PetriYN+1),A	;Set Petridish flag
	AND A
	CALL Z Border		;
	CALL VFlip64		;VFlip BMP
	JP Magnifyx4		;Magnify BMP
;========================================
;Macro-life "engine" by Toni Baker,
;originally published for the Spectrum in 
;ZX Computing Monthly, October 1986 and 
;modified for the ZX81 by Greg Harder.
;========================================
;Test surrounding cells
;========================================
@Ncell
	SBC HL,DE		;Up one pixel 
	LD A,H 			;A=High part of address
	CP 7F			;Test for underflow
	JR NZ Exit		;Junp if not off grid
	LD DE,0180		;Offset to bottom row
	ADD HL,DE		;Add offset
@ResetDE
	LD DE,0008		;Reset row counter
	JR Exit
@SCell
	ADD HL,DE 		;Down one pixel
	LD DE,817F		;Test for overflow
	EX DE,HL		;Swap DE & HL
;	AND A
	SBC HL,DE
	EX DE,HL		;Swap DE & HL
	JR NC ResetDE
	LD DE,0180		;Offset to top row
	AND A			;Clear CY
	SBC HL,DE		;Wrap to top row
	JR ResetDE
@WCell
	RLC C 			;Left one pixel (assuming within chr square)
	JR NC Exit		;Jump if within chr square
	LD A,L 			;A=Low part of address
	DEC A 			;Decrement to move left one chr
	JR WRAP
@ECell
	RRC C  			;Right one pixel (assuming within chr square)
	JR NC Exit		;Jump if within chr square
@Rite
	LD A,L 			;A=Low part of address
	INC A 			;Increment to move right one chr
@Wrap
	XOR L 			;Combine A and L by taking bits 7,6, and 5
	AND 07 			;  of L, and bits 4,3,2,1 and 0 of A.
	XOR L 			;  This has the effect of "joining" the
	LD L,A 			;  left and right hand edges of the screen.
@Exit
	LD A,C 			;A=Pixel position withing chr square
	AND (HL) 		;Test the specified screen pixel
	RET Z 			;Return if the pixel is OFF
	INC B 			;Otherwise increase cell count in B register
	RET
;========================================
;LIFE Engine
;========================================
@LIFE
	LD HL,GenX		;Start of GenX
	LD BC,0180 		;Number of bytes in GenX
	LD DE,GenCopy		;Start of GenCopy
	PUSH DE 		;PUSH GenCopy address
	LDIR 			;Copy GenX to GenCopy
	CALL Magnifyx4		;Display GenX
	POP HL 			;POP GenCopy address
	LD DE,0008 		;DE=8=Bytes/row
	AND A 			;Clear CY
	LD C,80 		;C has leftmost pixel set, thus HL and C
				;  together point to first pixel in GenCopy
@Life01
	LD B,D	 		;Initialize cell count to 0
	PUSH BC 		;PUSH pixel position
	PUSH HL 		;PUSH GenCopy address
	CALL NCell		;Count, in B, the number of cell which
	CALL ECell		;  surround the given pixel
	CALL SCell
	CALL SCell
	CALL WCell
	CALL WCell
	CALL NCell
	CALL NCell
	LD A,B 			;A=number of cells surrounding pixel
	POP HL 			;POP GenCopy address
	POP BC 			;POP pixel position
	SET 4,H			;Point HL into GenX (rather than GenCopy).
				;H=80 changes to H=90
	CP 02
	JR C Death		;Jump if cell count<2
	CP 04 
	JR NC Death		;Jump if cell count>3
	CP 03 
	JR NZ Static		;Jump if cell count=2
	LD A,C 			;A=pixel position
	OR (HL) 		;Prepare to set pixel to create cell
	JR Birth
@Death
	LD A,C			;A=Pixel position 
	CPL 
	AND (HL)		;Prepare to reset pixel to remove Cell.
@Birth
	LD (HL),A		;Set or reset pixel as required. 
@Static
	RES 4,H			;Point HL back into GenCopy
				;H=90 changes to H=80
	RRC C 			;Right one pixel assuming within pixel square
	JR NC Life01		;Jump back if within chr square
	CALL Rite		;Rite one pixel otherwise
	LD A,L 			;A=Low part of address
	AND 07 
	JR NZ Life01		;Jump back unless off right edge of the screen
	CALL SCell		;Down one pixel
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	RET NZ			;Return if keypressed
	LD A,H 			;A=High part of address
	XOR 80
	OR L 
	JR NZ Life01		;Jump back unless whole screen done
@PetriYN
	JR +3			;JR 00=Bounded JR +3=Unbounded
	CALL Border
	RET
;================================
; Clear Gen0 
;================================
@Clear
	LD BC,0180		;Bytes in GenX, 48*8
	LD HL,GenX		;Start of GenX
	LD (HL),FF		;Clear 1st byte
	LD DE,GenX+1
	LDIR
	RET
;========================================
; Negative (Invert Image)
;========================================
@InvBMP
	LD BC,1800		;Bytes in GenX, 192*32
	LD HL,HRDFile
	JR Invert+3
@Invert64
	LD BC,0180		;Bytes in GenX, 48*8
	JR Invert
@Invert128
	LD BC,0600		;Bytes in GenX, 96*16
;========================================
@Invert
	LD HL,GenX		;Start of GenX
	LD A,(HL)		;Get value
	CPL			;Invert pixels
	LD (HL),A		;Set new value
	INC HL			;Next GenX address
	DEC BC			;Decrease counter
	LD A,B 
	OR C 
	JR NZ Invert+3 		;Loop until BC=0
	RET
;========================================
; Vertical Flip BMP Image
;========================================
; Set flip parameters
;========================================
@FlipBMP
	LD BC,17E0 		;Offset to bottom row, 191*32
	LD D,60			;Row counter, 96
	LD A,20			;Bytes/row, 32
	LD HL,HRDFile		;Start of HRDFile
	JR VFlip+3
@VFlip64
	LD BC,0178 		;Offset to bottom row, 47*8
	LD D,18			;Row counter, 24
	LD A,08			;Bytes/row, 8
	JR VFlip
@VFlip128
	LD BC,05F0 		;Offset to bottom row, 95*16
	LD D,30			;Row counter,48
	LD A,10			;Bytes/row, 16
;========================================
@VFlip
	LD HL,GenX		;Start of GenX
	LD (RowCount+1),A	;Set bytes/row value
	LD (TopRow),HL		;Save top row addr
	ADD HL,BC		;HL=Address of bottom row.
	LD B,D			;B=Row counter
@VFLoop
	PUSH BC			;PUSH counter
	LD (BotRow),HL		;Save bottom row addr
	LD HL,(TopRow)		;Source, top row data
	LD DE,RowSave		;Destination, storage area
@RowCount
	LD BC,0000		;Bytes/row count
	LD A,C			;Save count in A
	LDIR			;Copy
	LD HL,(TopRow)		;Top row address
	EX DE,HL		;Destination, top row
	LD HL,(BotRow)		;Source, bottom row data
	LD C,A			;Restore counter
	LDIR			;Copy
	LD (TopRow),DE		;Save top row address
	LD HL,RowSave		;Source, top row data
	LD DE,(BotRow)		;Destination, bottom row
	LD C,A			;Restore counter
	LDIR			;Copy
	LD C,A			;Calculate address of next bottom row
	LD HL,(BotRow)
	XOR A			;Clear CY
	SBC HL,BC
	POP BC			;POP row counter
	DJNZ VFLoop		;Loop until B=0
	LD C,B			;Return 0
	RET
;========================================
; Magnify routine
; 
; Convert 64x48 BMP to 256x192
;========================================
@Magnifyx4
	LD BC,0830		;B=8=Column count, C=48=Row count
	LD DE,HRDFile		;HRDFile address
	LD HL,GenX		;GenX address
	LD (GenAddr+1),HL	;Set GenX line address
@MagLoop1
	PUSH BC			;PUSH counters
	PUSH DE			;PUSH HRDFile address
@GenAddr
	LD HL,0000		;Current GenX line address
@MagLoop2
	CALL BitTest		;Test bits for set or unset values
	CALL BitTest+1
	CALL BitTest+1
	CALL BitTest+1
	INC HL			;Next pattern address
	DJNZ MagLoop2
	LD (GenAddr+1),HL	;Set next GenX line
	POP HL			;POP HRDFile address
	LD A,03			;Duplicate row 3 times to make 4x4 pixels
@MagLoop3
	LD C,20			;Byte count, 32 Bytes/row
	LDIR			;Copy line
	DEC A
	JR NZ MagLoop3		;Loop until A=0
	POP BC			;Restore counters
	DEC C			;Decrease row counter
	JR NZ MagLoop1		;Loop until all rows done
	RET
;========================================
; Test bits for set or unset values
;========================================
@BitTest
	LD C,(HL)		;Get GenX value
	XOR A			;A=0
	RLC C			;Test Bit 7/5/3/1
	JR NC +2		;Jump if 0
	LD A,F0
	RLC C			;Test Bit 6/4/2/0
	JR NC +2		;Jump if 0
	ADD A,0F
	EX DE,HL		;Swap GenX/HRDFile addresses
	LD (HL),A		;Print pattern
	EX DE,HL		;Restore GenX/HRDFile addresses
	INC DE			;Next HRDFile address
	RET
;================================
; Upsize 32x24 to 64x48
;================================
@Upsize
	CALL Clear
	LD HL,(DFile)		;Start of DFile with 32x24 pattern
	INC HL
	LD DE,9062		;Gen0 address
	LD C,18			;Row count
@UpSz01
	LD B,04			;Column counter, 4*8 bytes
@UpSz02
	PUSH BC			;PUSH counters
	PUSH DE			;PUSH Gen0 addr
	LD B,08			;Counter, 8 bytes
	LD D,00			;Bit pattern holder
@UpSz03
	LD E,01			;CY value
	LD A,(HL)		;Get value, " " or "O"
	AND A			;Test value, CY=0
	JR Z +2			;Jump if 0
	RR E			;Set CY
	RL D			;CY to D, 0 if " ", 1 if "O"
	INC HL			;Next DFile addr
	DJNZ UpSz03		;Loop until B=0=8 bytes processed
	LD A,D			;A=Bit pattern
	CPL			;Invert the pattern
	POP DE			;POP destination
	LD (DE),A		;Set pattern
	INC DE			;Next destination
	POP BC			;POP counters
	DJNZ UpSz02		;Loop until B=0=DFile row complete
	INC HL			;Next DFile row
	INC DE			;Next Gen0 row
	INC DE
	INC DE
	INC DE
	DEC C			;Decrease row counter
	JR NZ UpSz01		;Loop until C=0
	JP VFlip64		;VFlip the pattern
;========================================
; Border 64x48
;========================================
@Border
	XOR A			;Erase border
;	LD A,FF			;Draw border
	LD HL,GenX		;NW corner
	LD (HL),A		;Source value
	LD DE,GenX+1
	LD BC,0007		;Column count-1
	LDIR			;Draw top row
	LD B,+47		;Row count-1
	LD DE,0008		;Offset to next row
	ADD HL,DE		;Right margin next row
	RES 0,(HL)		;Erase right margin
;	SET 0,(HL)		;Draw right margin
	DJNZ -5			;Loop until B=0
	LD (HL),A		;Source value, SE corner
	LD D,H
	LD E,L
	DEC DE			;Set destination
	LD C,07			;Column count-1
	LDDR			;Draw bottom row
	LD DE,0008		;Offset to next row
	LD B,+47		;Row count-1
	SBC HL,DE		;Left margin next row
	RES 7,(HL)		;Erase left margin
;	SET 7,(HL)		;Draw left margin
	DJNZ -6			;Loop until B=0
	RET			;Ret 0
;================================
; Load BMP Hdr
;================================
@BMPHDR
	LD HL,Header		;Set source
	LD DE,9FC2		;Set destination
	LD BC,003E		;Byte count
	LDIR			;Copy header data
	RET
;================================
; BMP Header data
;================================
@Header
	42 4D 3E 18 00 00 00 00	00 00 3E 00 00 00 28 00
	00 00 00 01 00 00 C0 00	00 00 01 00 01 00 00 00
	00 00 00 18 00 00 C4 0E	00 00 C4 0E 00 00 00 00
	00 00 00 00 00 00 00 00 00 00 FF FF FF 00
;Buffer Bytes
	00 00 00 00 00 00 00 00
	00 00 00 00 00 00 
