;========================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;========================================
; 128x96 LIFE
; Based on "Macro-LIFE" by Toni Baker
; from ZX Computing, 10/1986 p.54
;========================================

ORG 4A65			;19045

@Versn=4009			;16393 Border/No border flag
@LastK=4025			;Keyboard debounce value
@GenX=9000			;Start of GenX
@GenCopy=8000			;Start of GenCopy
@HRDFILE=A000			;Start HRDFile
@BMPHeader=4803
@InvBMP=4806
@FlipBMP=4809
@Invert128=480F
@VFlip128=4818

	JR Gen0			;19045
	JR LIFE			;19047
	JP Clear		;19049
	JP UpSize		;19051
;========================================
; Prep Gen0 for use
;========================================
@Gen0
	CALL Invert128		;Invert BMP
	LD A,(Versn)		;Get Petridish flag
	LD (PetriYN+1),A	;Set Petridish flag
	AND A
	CALL Z Border		;
	CALL VFlip128		;VFlip BMP
	JP Magnifyx2		;Magnify BMP
;========================================
;Macro-life "engine" by Toni Baker,
;originally published for the Spectrum in 
;ZX Computing Monthly, October 1986 and 
;modified for the ZX81 by Greg Harder.
;========================================
;Test surrounding cells
;========================================
@Ncell
	SBC HL,DE		;Up one pixel 
	LD A,H 			;A=High part of address
	CP 7F			;Top row off grid?
	JR NZ Exit		;Junp if not off grid
	LD H,85 		;Wrap to bottom row
	JR Exit
@SCell
	ADD HL,DE 		;Down one pixel
	LD A,H 			;A=High part of address
	CP 86			;Bottom row off grid?
	JR NZ Exit		;Jump if not off grid
	LD H,80			;Wrap to top row
	JR Exit
@WCell
	RLC C 			;Left one pixel (assuming within chr square)
	JR NC Exit		;Jump if within chr square
	LD A,L 			;A=Low part of address
	DEC A 			;Decrement to move left one chr
	JR WRAP
@ECell
	RRC C  			;Right one pixel (assuming within chr square)
	JR NC Exit		;Jump if within chr square
@Rite
	LD A,L 			;A=Low part of address
	INC A 			;Increment to move right one chr
@Wrap
	XOR L 			;Combine A and L by taking bits 7,6, and 5
	AND 0F 			;  of L, and bits 4,3,2,1 and 0 of A.
	XOR L 			;  This has the effect of "joining" the
	LD L,A 			;  left and right hand edges of the screen.
@Exit
	LD A,C 			;A=Pixel position withing chr square
	AND (HL) 		;Test the specified screen pixel
	RET Z 			;Return if the pixel is OFF
	INC B 			;Otherwise increase cell count in B register
	RET
;========================================
;LIFE Engine
;========================================
@LIFE
	LD HL,GenX		;Start of GenX
	LD DE,GenCopy		;Start of GenCopy
	LD BC,600 		;Number of bytes in GenX
	PUSH DE 		;PUSH GenCopy address
	LDIR 			;Copy GenX to GenCopy
	CALL Magnifyx2		;Display GenX
	POP HL 			;POP GenCopy address
	LD DE,0010 		;DE=16=Bytes/row
	AND A 			;Clear CY
	LD C,80 		;C has leftmost pixel set, thus HL and C
				;  together point to first pixel in GenCopy
@Life01
	LD B,00 		;Initialize cell count to 0
	PUSH BC 		;PUSH pixel position
	PUSH HL 		;PUSH GenCopy address
	CALL NCell		;Count, in B, the number of cell which
	CALL ECell		;  surround the given pixel
	CALL SCell
	CALL SCell
	CALL WCell
	CALL WCell
	CALL NCell
	CALL NCell
	LD A,B 			;A=number of cells surrounding pixel
	POP HL 			;POP GenCopy address
	POP BC 			;POP pixel position
	SET 4,H			;Point HL into GenX (rather than GenCopy).
				;H=80 changes to H=90
	CP 02
	JR C Death		;Jump if cell count<2
	CP 04 
	JR NC Death		;Jump if cell count>3
	CP 03 
	JR NZ Static		;Jump if cell count=2
	LD A,C 			;A=pixel position
	OR (HL) 		;Prepare to set pixel to create cell
	JR Birth
@Death
	LD A,C			;A=Pixel position 
	CPL 
	AND (HL)		;Prepare to reset pixel to remove Cell.
@Birth
	LD (HL),A		;Set or reset pixel as required. 
@Static
	RES 4,H			;Point HL back into GenCopy
				;H=90 changes to H=80
	RRC C 			;Right one pixel assuming within pixel square
	JR NC Life01		;Jump back if within chr square
	CALL Rite		;Rite one pixel otherwise
	LD A,L 			;A=Low part of address
	AND 0F 
	JR NZ Life01		;Jump back unless off right edge of the screen
	CALL SCell		;Down one pixel
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	RET NZ			;Return if keypressed
	LD A,H 			;A=High part of address
	XOR 80
	OR L 
	JR NZ Life01		;Jump back unless whole screen done
@PetriYN
	JR +3			;JR 00=Bounded JR +3=Unbounded
	CALL Border
	RET
;========================================
; Magnify routine
;========================================
@Magnifyx2
	LD DE,HRDFile		;HRDFile address
	LD HL,GenX		;GenX address
	LD (GenAddr+1),HL	;Set GenX line address
	LD BC,1060		;B=16=Column count, C=96=Row count
@Mag2
	PUSH BC			;PUSH counters
	PUSH DE			;PUSH HRDFile address
@GenAddr
	LD HL,0000		;Current GenX line address
@Mag3
	CALL BitTest
	CALL BitTest+1
	INC HL			;Next pattern address
	DJNZ Mag3
	LD (GenAddr+1),HL	;Set next GenX line
	POP HL			;POP HRDFile address
	LD C,20			;Byte count
	LDIR			;Copy line
	POP BC			;Restore counters
	DEC C			;Decrease row counter
	JR NZ Mag2		;Loop until all rows done
	RET
;========================================
; Test bits for set or unset values
;========================================
@BitTest
	LD C,(HL)		;Get GenX value
	XOR A			;A=0
	RLC C			;Test Bit 7/3
	JR NC +4		;Jump if 0
	SET 7,A			;Set bits
	SET 6,A
	RLC C			;Test Bit 6/2
	JR NC +4		;Jump if 0
	SET 5,A			;Set bits
	SET 4,A
	RLC C			;Test Bit 5/1
	JR NC +4		;Jump if 0
	SET 3,A			;Set bits
	SET 2,A
	RLC C			;Test Bit 4/0
	JR NC +4		;Jump if 0
	SET 1,A			;Set bits
	SET 0,A
	EX DE,HL		;Swap GenX/HRDFile addresses
	LD (HL),A		;Print pattern
	EX DE,HL		;Restore GenX/HRDFile addresses
	INC DE			;Next HRDFile address
	RET
;========================================
; Border 128x96
;========================================
@Border
	XOR A			;Erase border
;	LD A,FF			;Draw border
	LD HL,GenX		;Set source, NW corner
	LD (HL),A		;Source value
	LD DE,GenX+1		;Set destination
	LD BC,000F		;Column count-1
	LDIR			;Draw top row
	LD B,+95		;Row count-1
	LD DE,0010		;Offset to next row
	ADD HL,DE		;Right margin next row
	RES 0,(HL)		;Erase right margin
;	SET 0,(HL)		;Draw right margin
	DJNZ -5			;Loop until B=0
	LD (HL),A		;Source value, SE corner
	LD D,H
	LD E,L
	DEC DE			;Set destination
	LD C,0F			;Column count-1
	LDDR			;Draw bottom row
	LD DE,0010		;Offset to next row
	LD B,+95		;Row count-1
	SBC HL,DE		;Left margin next row
	RES 7,(HL)		;Erase left margin
;	SET 7,(HL)		;Draw left margin
	DJNZ -6			;Loop until B=0
	RET			;Ret 0
;================================
; Clear Gen0 
;================================
@Clear
	LD BC,0600		;Bytes in GenX, 96*16
	LD HL,GenX		;Set source, start of GenX
	LD (HL),FF		;Clear 1st byte
	LD DE,GenX+1		;Set destination
	LDIR			;Clear BC bytes
	RET			;Ret 0
;================================
; Upsize 64x48 to 128x96
;================================
@Upsize
	LD A,+48		;Row count
	LD HL,GenX		;Set source, start of GenX
	LD DE,9184		;Set destination
	LD BC,0008		;Byte count
	LDIR			;Copy BMP line
	EX DE,HL
	LD C,08			;Byte count
	ADD HL,BC		;Next destination line
	EX DE,HL
	DEC A			;Decrease row counter
	JR NZ -10
	LD BC,017F		;Clear the 64x48 BMP data
	JR Clear+3
;Buffer bytes
	00 00 00 00		;Some buffer bytes
	00 00 