;========================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;========================================
; 256x192 LIFE
; Based on "Macro-LIFE" by Toni Baker
; from ZX Computing, 10/1986 p.54
;========================================

ORG 4BCC			;19404

@Versn=4009			;16393 Border/No border flag
@LastK=4025			;Keyboard debounce value
@InvBMP=4806			;18438
@FlipBMP=4809			;18441
@HRDFILE=A000			;Start of HRDFile
@GenX=8000			;Start of GenX

	JR Gen0			;19404
	JR LIFE			;19406
	JP Clear		;19408
	JP UpSize		;19411
;========================================
; Prep Gen0 for use
;========================================
@Gen0
	CALL InvBMP		;Invert BMP
	LD A,(Versn)		;Get Petridish flag
	LD (PetriYN+1),A	;Set Petridish flag
	AND A
	CALL Z Border
	JP FlipBMP		;Flip BMP
;========================================
;Macro-life "engine" by Toni Baker,
;originally published for the Spectrum in 
;ZX Computing Monthly, October 1986 and 
;modified for the ZX81 by Greg Harder.
;========================================
;Test surrounding cells
;========================================
@Ncell
	SBC HL,DE		;Up one pixel 
	LD A,H 			;A=High part of address
	CP 7F			;Top row off grid?
	JR NZ Exit		;Junp if not off grid
	LD H,97 		;Wrap to bottom row
	JR Exit
@SCell
	ADD HL,DE 		;Down one pixel
	LD A,H 			;A=High part of address
	CP 98			;Bottom row off grid?
	JR NZ Exit		;Jump if not off grid
	LD H,80			;Wrap to top row
	JR Exit
@WCell
	RLC C 			;Left one pixel (assuming within chr square)
	JR NC Exit		;Jump if within chr square
	LD A,L 			;A=Low part of address
	DEC A 			;Decrement to move left one chr
	JR WRAP
@ECell
	RRC C  			;Right one pixel (assuming within chr square)
	JR NC Exit		;Jump if within chr square
@Rite
	LD A,L 			;A=Low part of address
	INC A 			;Increment to move right one chr
@Wrap
	XOR L 			;Combine A and L by taking bits 7,6, and 5
	AND 1F 			;  of L, and bits 4,3,2,1 and 0 of A.
	XOR L 			;  This has the effect of "joining" the
	LD L,A 			;  left and right hand edges of the screen.
@Exit
	LD A,C 			;A=Pixel position withing chr square
	AND (HL) 		;Test the specified screen pixel
	RET Z 			;Return if the pixel is OFF
	INC B 			;Otherwise increase cell count in B register
	RET
;========================================
;LIFE Engine
;========================================
@LIFE
	LD HL,HRDFile		;Screen
	LD DE,GenX		;Screen copy
	LD BC,1800 		;Number of bytes in screen
	PUSH DE 		;PUSH starting address of GenX
	LDIR 			;Make copy of screen
	POP HL 			;POP starting address of GenX
	LD DE,0020 		;DE=Bytes/pixel row
	AND A 			;Clear CY
	LD C,80 		;C has leftmost pixel set, thus HL and C
				;  together point to first pixel in GenX
@Life01
	LD B,00 		;Initialize cell count to 0
	PUSH BC 		;PUSH pixel position
	PUSH HL 		;PUSH screen-copy address
	CALL NCell		;Count, in B, the number of cell which
	CALL ECell		;  surround the given pixel
	CALL SCell
	CALL SCell
	CALL WCell
	CALL WCell
	CALL NCell
	CALL NCell
	LD A,B 			;A=number of cells surrounding pixel
	POP HL 			;POP screen-copy address
	POP BC 			;POP pixel position
	SET 5,H			;Point HL into screen (rather than copy).
				;H=80 changes to H=A0
	CP 02
	JR C Death		;Jump if cell count<2
	CP 04 
	JR NC Death		;Jump if cell count>3
	CP 03 
	JR NZ Static		;Jump if cell count=2
	LD A,C 			;A=pixel position
	OR (HL) 		;Prepare to set pixel to create cell
	JR Birth
@Death
	LD A,C			;A=Pixel position 
	CPL 
	AND (HL)		;Prepare to reset pixel to remove Cell.
@Birth
	LD (HL),A		;Set or reset pixel as required. 
@Static
	RES 5,H			;Point HL back into screen copy.
				;H=A0 changes to H=80
	RRC C 			;Right one pixel assuming within pixel square
	JR NC Life01		;Jump back if within chr square
	CALL Rite		;Rite one pixel otherwise
	LD A,L 			;A=Low part of address
	AND 1F 
	JR NZ Life01		;Jump back unless off right edge of the screen
	CALL SCell		;Down one pixel
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	RET NZ			;Return if keypressed
	LD A,H 			;A=High part of address
	XOR 80
	OR L 
	JR NZ Life01		;Jump back unless whole screen done
@PetriYN
	JR +3			;JR 00=Bounded JR +3=Unbounded
	CALL Border
	RET
;========================================
; Border 256x192
;========================================
@Border
	XOR A			;Erase border
;	LD A,FF			;Draw border
	LD HL,HRDFile		;NW corner
	LD (HL),A		;Source value
	LD DE,HRDFile+1
	LD BC,001F		;Column count-1
	LDIR			;Draw top row
	LD B,+191		;Row count-1
	LD DE,0020		;Offset to next row
	ADD HL,DE		;Right margin next row
	RES 0,(HL)		;Erase right margin
;	SET 0,(HL)		;Draw right margin
	DJNZ -5			;Loop until B=0
	LD (HL),A		;Source value, SE corner
	LD D,H
	LD E,L
	DEC DE			;Set destination
	LD C,1F			;Column count-1
	LDDR			;Draw bottom row
	LD DE,0020		;Offset to next row
	LD B,+191		;Row count-1
	SBC HL,DE		;Left margin next row
	RES 7,(HL)		;Erase left margin
;	SET 7,(HL)		;Draw left margin
	DJNZ -6			;Loop until B=0
	RET			;Ret 0
;================================
; Clear Gen0 
;================================
@Clear
	LD BC,1800		;Bytes in HRDFile
	LD HL,HRDFile		;Set source, start of HRDFile
	LD (HL),FF		;Clear 1st byte
	LD DE,HRDFile+1		;Set destination
	LDIR			;Clear BC bytes
	RET			;Ret 0
;================================
; Upsize 128x96 to 256x192
; LOAD BMP to 80C2/32962
;================================
@Upsize
	LD A,+96		;Row count
	LD HL,8100		;Set source, start of BMP data
	LD DE,A608		;Set destination, HRDFile
	LD BC,0010		;Byte count
	LDIR			;Copy BMP line
	EX DE,HL
	LD C,10			;Byte count
	ADD HL,BC		;Next destination line
	EX DE,HL
	DEC A			;Decrease row counter
	JR NZ -10
	LD C,A			;BC=0
	RET			;Ret 0
	00