;========================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;========================================
; WRX Video Driver by Wilf Rigter
;========================================

ORG 45CF			;17871

@PushReg=0220			;PUSH REGs then start DISPLAY-1
@Display3=0292			;DISPLAY BLANK LINES routine
@PopReg2=02A4			;POP REGs and RETURN to BASIC
@Break1=0F46			;BREAK routine
@Video1=0281			;Restart standard video display
@Frames=4034			;FRAME Counter
@Slow=0F2B			;SLOW command
@HRDFile=A000			;Start of Hi-Res D-File

;==========================
; Jump Table
;==========================
	JR WRXOn		;17963 Turn WRX Mode on
	JR WRXOff		;17965 Turn WRX Mode off
	JP WRXCls		;17967 Clear the HRDFile
	JP WRXInv		;17969 Soft Invert of HRDFile
;========================================
; Wilf Rigter's Hi-Res display driver
;========================================
@WRX
	LD IX,HR		;Starts the hires mode when JP (IX) is made
	RET
@HLine
	LD R,A 			;Load LSB to R register which is RFSH address LSB
	00 00 00 00		;32 NOPs=32x8 bits=256 pixels each HLINE
	00 00 00 00
	00 00 00 00
	00 00 00 00
	00 00 00 00
	00 00 00 00
	00 00 00 00
	00 00 00 00
	RET NZ 			;Always returns because Z flag=0
;========================================
@HR
	LD B,06			;Delay sets the left edge of the screen 
@HR0
	DJNZ HR0		;Delay loop
	00 00 00 00 00		;Fine tuning
	DEC B			;Make Z flag=0
	LD HL,HRDFile 		;HL is the HRDFile address pointer for each hline
				;HL should be divisible by 32
	LD DE,0020 		;32 bytes offset is added to HL for next hline
	LD B,C0	 		;192 lines per hires screen  
@HRLoop1
	LD A,H 			;Get HRDFile address MSB from HL
	LD I,A 			;Load MSB into I register which is RFSH address MSB
	LD A,L 			;Get HRDFile address LSB from HL
@SetHLine
	CALL 0000 		;CALL HLINE + 8000
	ADD HL,DE 		;Add 32 to HL to point to next hline
	DEC B 			;Decrement line counter
	JP NZ HRLoop1 		;Test for last line
@HR2
	CALL Display3 		;Return to application program
	CALL PushReg 		;First PUSH register, then do VSYNC and get KEYBD
	CALL Break1 		;Check break (space) key
	LD A,1E			;The I register is restored with the MSB address of
	LD I,A 			;of the ROM pattern table in case of BREAK key down
	JR NC +4		;IF break then restore Sinclair video
	LD IX,HR 		;Reload HR vector if no BREAK
@HR3
	JP PopReg2		;now return to BASIC and other application programs
;================================
; Start WRX Mode
;================================
@WRXOn
	CALL Slow
	LD HL,HLine		;Adjust HLine pointer
	LD BC,8000
	ADD HL,BC
	LD (SetHLine+1),HL	;Set HLine pointer
	JR WRX
;================================
; Restore Sinclair display driver
;================================
@WRXOff
	HALT 			;Sync the display
	LD A,(Frames) 		;Get Frames
	LD C,A 
	LD A,(Frames) 		;Get Frames
	CP C 
	JR Z -6
	LD A,1E 		;Restore Sinclair video
	LD I,A 
	LD IX,Video1 		;Set IX vector to Sinclair display
	LD BC,0000 		;Return 0 to BASIC
	RET
;========================================
; Clear screen
;========================================
@WRXCls
	LD HL,HRDFile		;HRDFile address
	LD BC,17FF		;Byte count
	XOR A			;White screen
;	LD A,C			;Black screen
	LD (HL),A 		;Set value
	LD D,H
	LD E,L 
	INC DE 			;HRDFile+1 address 
	LDIR 			;Clear the rest
	RET			;Return 0 to BASIC
;========================================
; Soft Invert of HRDFile
;========================================
@WRXInv
	LD B,1F 		;Byte count
	LD HL,HLine+2 		;Line buffer address
	LD A,(HL) 		;Get current value
	ADD A,80 		;Invert value
	LD (HL),A		;Set new value
	INC HL			;Net line buffer address
	DJNZ WRXInv+5		;Loop until B=0
	LD C,B			;Return 0 to BASIC
	RET
